--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Calendar_Broker" );

-- Constants
CORECALENDAR_SECPERDAY = 86400;

--Global Variables
CalendarBroker.events = {
    EventsTodayCount = 0,
    EventsToday = {},
    EventsUpcoming = {},
};

function CalendarBroker.Time2Text( t )
    local days   = math.floor( t / 86400 );
    local hours  = math.floor( ( t - 86400 * days ) / 360 ) / 10;
    local minute = ( t - 86400 * days - math.floor( hours ) * 3600 ) / 60;
    local textT  = "ERROR";

    hours = hours + days * 86400;
    if( t <= 0 ) then
        textT = L["NOW"];
    elseif( days > 2 ) then
        textT = L["%i days"]:format( days );
    elseif( days == 1 ) then
        textT = L["1 day"];
    elseif( hours > 1.0 ) then
        textT = L["%.1f hours"]:format( hours );
    elseif( hours == 1 ) then
        textT = L["1 hour"];
    elseif( minute > 1 ) then
        textT = L["%.1f mins"]:format( minute );
    else
        textT = L["soon"];
    end--if
    return textT;
end--CalendarBroker.Time2Text( d, h, m )

function CalendarBroker:GetFormattedDate( seconds )
    if( self.db.profile.DateFormat_US ) then
        return date( "%m/%d/%y", seconds );
    end--if
    return date( "%d/%m/%y", seconds );
end--CalendarBroker:GetFormattedDate( seconds )

function CalendarBroker:GetFormattedTime( seconds )
    if( self.db.profile.TimeFormat_12h ) then
        return date( "%I:%M %p", seconds );
    end--if
    return date( "%H:%M", seconds );
end--CalendarBroker:GetFormattedDate( seconds )

function CalendarBroker.SecondsFromEpoch( month, day, year, hour, min, sec )
    local tim;
    
    if( type( month ) == "table" ) then
        tim = month;
    else
        tim = { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour or 0, ["min"] = min or 0, ["sec"] = sec or 0, };
    end--if
    return time( tim );
end--CalendarBroker.SecondsFromEpoch( month, day, year, hour, min, sec )

function CalendarBroker.PrintDate( seconds )
    -- Used to debug the math
    return date( "%m/%d/%y %H:%M:%S", seconds );
end--CalendarBroker.PrintDate( seconds )

function CalendarBroker:PrintDebug( text )
    if( self.db.profile.DebugMessages ) then
        self:Print( "|cffffff00DEBUG |cffffffff" .. text )
    end--if
end--CalendarBroker:PrintDebug( text )

local EVENT_TYPE_MAP = {
    [CALENDAR_EVENTTYPE_RAID or 1] = (" (%s)"):format( L["Raid"] ),
    [CALENDAR_EVENTTYPE_DUNGEON or 2] = (" (%s)"):format( L["Dungeon"] ),
    [CALENDAR_EVENTTYPE_PVP or 3] = (" (%s)"):format( L["PVP"] ),
    [CALENDAR_EVENTTYPE_MEETING or 4] = (" (%s)"):format( L["Meeting"] ),
    [CALENDAR_EVENTTYPE_OTHER or 5] = "",
};

local INVITE_TYPE_MAP = {
    [CALENDAR_INVITESTATUS_INVITED or 1] = (" (%s)"):format( L["Invited"] ),
    [CALENDAR_INVITESTATUS_ACCEPTED or 2] = (" (%s)"):format( L["Accepted"] ),
    [CALENDAR_INVITESTATUS_DECLINED or 3] = (" (%s)"):format( L["Declined"] ),
    [CALENDAR_INVITESTATUS_CONFIRMED or 4] = (" (%s)"):format( L["Confirmed"] ),
    [CALENDAR_INVITESTATUS_OUT or 5] = (" (%s)"):format( L["Out"] ),
    [CALENDAR_INVITESTATUS_STANDBY or 6] = (" (%s)"):format( L["Stand by"] ),
};

function CalendarBroker:ProcessWorldEvents()
    local currTime_Obj = date( "*t" );
    local currTime_Sec  = time( currTime_Obj );
    local ourevents = {};
    local counts = {};
    local _, month, _, year = CalendarGetDate();
    local prevmonth, prevyear, _, _ = CalendarGetMonth();

    CalendarSetAbsMonth( month, year );
    CalendarSetMonth( -1 );

    self.events.EventsTodayCount = 0;
    self.events.EventsToday = {};
    self.events.EventsUpcoming = {};

    for monthOffset = 0, 3 do
        CalendarSetMonth( 1 );

        local month, year, days, firstday = CalendarGetMonth();

        for day = 1, days do
            for i = 1, CalendarGetNumDayEvents( 0, day ) do
                local title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus, invitedBy, difficulty = CalendarGetDayEvent( 0, day, i );

                if( ( calendarType == "RAID_RESET" ) or ( calendarType == "RAID_LOCKOUT" ) ) then
                    if( difficulty > 1 ) then
                        table.insert( ourevents, { ["title"] = ("%s (%s)"):format( title, _G["DUNGEON_DIFFICULTY" .. difficulty] ), ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 3600, title, ["type"] = "RAID", } );
                    else
                        table.insert( ourevents, { ["title"] = title, ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 3600, title, ["type"] = "RAID", } );
                    end--if
                elseif( calendarType == "HOLIDAY" ) then
                    local event = ourevents[title .. tostring( counts[title] or 0 )];
                    local location = texture:match( "Calendar_DarkmoonFaire(%w+)" );

                    if( sequenceType == "START" ) then
                        ourevents[title .. tostring( counts[title] or 0 )] = { ["title"] = title, ["start"] = { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, }, ["length"] = ( 24 - hour ) * 3600 + ( 60 - minute ) * 60, ["type"] = calendarType, };
                        if( location ) then
                            if( location == "Elwynn" ) then
                                ourevents[title .. tostring( counts[title] or 0 )].title = ("%s (%s)"):format( title, L["Elwynn Forest"] );
                            elseif( location == "Mulgore" ) then
                                ourevents[title .. tostring( counts[title] or 0 )].title = ("%s (%s)"):format( title, L["Mulgore"] );
                            elseif( location == "Terokkar" ) then
                                ourevents[title .. tostring( counts[title] or 0 )].title = ("%s (%s)"):format( title, L["Terokkar Forest"] );
                            end--if
                        end--if
--                        self:Print( ("CalendarGetDayEvent() = %s, %i, %i, %s, %s, %i, %s, %s, %i, %s, %i"):format( title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus, invitedBy, difficulty ) );
                    elseif( sequenceType == "END" ) then
                        if( event ) then
                            event.length = event.length + hour * 3600 + minute * 60;
                        else
                            ourevents[title .. tostring( counts[title] or 0 )] = { ["title"] = title, ["start"] = { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, }, ["length"] = ( 24 - hour ) * 3600 + ( 60 - minute ) * 60, ["type"] = calendarType, };
                        end--if
                        counts[title] = ( counts[title] or 0 ) + 1;
                    else
                        -- Not sure what to do about "INFO"
                        if( event ) then
                            event.length = event.length + 24 * 3600;
                        else
                            ourevents[title .. tostring( counts[title] or 0 )] = { ["title"] = title, ["start"] = { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, }, ["length"] = 3600 * 2, ["type"] = calendarType, };
                            if( sequenceType ~= "ONGOING" ) then
                                counts[title] = ( counts[title] or 0 ) + 1;
                            end--if
                        end--if
                    end--if
                elseif( ( calendarType == "PLAYER" ) and ( inviteStatus ~= CALENDAR_INVITESTATUS_DECLINED ) and ( inviteStatus ~= CALENDAR_INVITESTATUS_OUT ) ) then
                    table.insert( ourevents, { ["title"] = title .. EVENT_TYPE_MAP[eventType] .. INVITE_TYPE_MAP[inviteStatus], ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 1 * 3600, title, ["type"] = calendarType, } );
                elseif( ( calendarType == "GUILD" ) and ( inviteStatus ~= CALENDAR_INVITESTATUS_DECLINED ) and ( inviteStatus ~= CALENDAR_INVITESTATUS_OUT ) ) then
                    table.insert( ourevents, { ["title"] = title .. EVENT_TYPE_MAP[eventType] .. INVITE_TYPE_MAP[inviteStatus], ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 1 * 3600, title, ["type"] = calendarType, } );
                elseif( calendarType == "SYSTEM" ) then
                    table.insert( ourevents, { ["title"] = title, ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 1 * 3600, title, ["type"] = calendarType, } );
                elseif( calendarType == "ARENA" ) then
                    table.insert( ourevents, { ["title"] = title, ["start"] =  { ["year"] = year, ["month"] = month, ["day"] = day, ["hour"] = hour, ["min"] = minute, ["sec"] = 0, } , ["length"] = 1 * 3600, title, ["type"] = calendarType, } );
                end--if
            end--for
        end--for
    end--for

    CalendarSetAbsMonth( prevmonth, prevyear );
    for _, event in pairs( ourevents ) do
        self:AddEvent( time( event.start ), event.length, event.title, event.type, "WOW" );
    end--for

    -- Since different OS and different timezones may have different points of Epoch,
    -- everything is referenced to the release of WOW (Azeroth's epoch)
    self.db.profile.ZuluOffset = self.SecondsFromEpoch( 11, 23, 2004 );

    -- If in battleground may need to offset servertime
    local ServerHour, ServerMin = GetGameTime();
    local BGflag = false;

    for i = 1, MAX_BATTLEFIELD_QUEUES do
        local status, _, _, _, _, _ = GetBattlefieldStatus( i );

         BGflag = ( status == "active" ) or BGflag;
    end--for
    if( BGflag ) then
        ServerHour = currTime_Obj.hour - self.db.profile.ServerTimeOffset;
    else
        self.db.profile.ServerTimeOffset = currTime_Obj.hour - ServerHour;
    end--if

    -- if GEM is loaded, then add the events that I am subscribed to
    if( GEM3_Events == nil ) then
        self.db.profile.UseGEMdatabase = false;
    end--if

    if( self.db.profile.UseGEMdatabase and GEM3_Events[GEM3_Realm] ) then
        for i, event in ipairs( GEM3_Events[GEM3_Realm] ) do
            local EVloc = event.ev_place;
            local EVsec = event.ev_date + self.db.profile.ServerTimeOffset * 3600;
            local ET = date( "*t", EVsec );
            local EVtype = event.ev_type;
            local type = "GEM";

            if( EVtype == GEM3_EVT_TYPE_NONE ) then
                type = "OTHER";
            elseif( EVtype == GEM3_EVT_TYPE_TEST ) then
                type = "OTHER";
            elseif( EVtype == GEM3_EVT_TYPE_RESET ) then
                type = "RAID";
            elseif( EVtype == GEM3_EVT_TYPE_LFG ) then
                type = "PLAYER";
            elseif( EVtype == GEM3_EVT_TYPE_POLL ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_PLAYER_AVALABILITY ) then
                type = "PLAYER";
            elseif( EVtype == GEM3_EVT_TYPE_RAID_AVAILABILITY ) then
                type = "RAID";
            elseif( EVtype == GEM3_EVT_TYPE_INSTANCE5 ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_INSTANCE5_BC ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_INSTANCE5_WOTLK ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_INSTANCE5_BCH ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_INSTANCE5_WOTLKH ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_RAID ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_RAID_BC ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_RAID_WOTLK ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_PVP ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_GUILD_EVENT ) then
                type = "GUILD";
            elseif( EVtype == GEM3_EVT_TYPE_GUILD_OFFICERMEETING ) then
                type = "GUILD";
            end--if
            self:AddEvent( EVsec, 3600, EVloc, type, "GEM" );
        end--for
    end--if

    -- Sort the data
    table.sort( self.events.EventsToday, function( a, b ) return a[4] < b[4] end );
    table.sort( self.events.EventsUpcoming, function( a, b ) return a[1] < b[1] end );
end--CalendarBroker:ProcessWorldEvents()

function CalendarBroker:AddEvent( start, duration, text, type )
    -- start and duration are in units of seconds
    local current = time();
    local EndTime = start + duration;

    if( not type ) then
        type = "NONE";
    end--if
--    self:PrintDebug( "Adding event " .. text .. " " .. date( "%m/%d/%y %H:%M:%S", start ) .. " to " .. date( "%m/%d/%y %H:%M:%S", EndTime ) );

    if( current < EndTime ) then
        -- Has not ended
        if( current >= start ) then
            table.insert( self.events.EventsToday, { 0, text, type, EndTime, calendar, } );
            self.events.EventsTodayCount = self.events.EventsTodayCount + 1;
--            self:PrintDebug( "Current event" );
        else
            -- Compute the number of days, not seconds
            table.insert( self.events.EventsUpcoming, { start - current, text, type, start, calendar, } );
--            self:PrintDebug( "Future event " );
        end--if
--    else
--        self:PrintDebug( "Already over" );
    end--if
end--CalendarBroker:AddEvent( start, duration, text )
